@@ -2,17 +2,15 @@
 
 package com.metallic.chiaki.settings
 
-import android.view.View
+import android.view.LayoutInflater
 import android.view.ViewGroup
 import androidx.recyclerview.widget.RecyclerView
-import com.metallic.chiaki.R
 import com.metallic.chiaki.common.RegisteredHost
-import com.metallic.chiaki.common.ext.inflate
-import kotlinx.android.synthetic.main.item_registered_host.view.*
+import com.metallic.chiaki.databinding.ItemRegisteredHostBinding
 
 class SettingsRegisteredHostsAdapter: RecyclerView.Adapter<SettingsRegisteredHostsAdapter.ViewHolder>()
 {
-	class ViewHolder(itemView: View): RecyclerView.ViewHolder(itemView)
+	class ViewHolder(val binding: ItemRegisteredHostBinding): RecyclerView.ViewHolder(binding.root)
 
 	var hosts: List<RegisteredHost> = listOf()
 		set(value)
@@ -21,15 +19,15 @@ class SettingsRegisteredHostsAdapter: RecyclerView.Adapter<SettingsRegisteredHos
 			notifyDataSetChanged()
 		}
 
-	override fun onCreateViewHolder(parent: ViewGroup, viewType: Int) = ViewHolder(parent.inflate(R.layout.item_registered_host))
+	override fun onCreateViewHolder(parent: ViewGroup, viewType: Int)
+		= ViewHolder(ItemRegisteredHostBinding.inflate(LayoutInflater.from(parent.context), parent, false))
 
 	override fun getItemCount() = hosts.size
 
 	override fun onBindViewHolder(holder: ViewHolder, position: Int)
 	{
-		val view = holder.itemView
 		val host = hosts[position]
-		view.nameTextView.text = "${host.serverNickname} (${if(host.target.isPS5) "PS5" else "PS4"})"
-		view.summaryTextView.text = host.serverMac.toString()
+		holder.binding.nameTextView.text = "${host.serverNickname} (${if(host.target.isPS5) "PS5" else "PS4"})"
+		holder.binding.summaryTextView.text = host.serverMac.toString()
 	}
 }
\ No newline at end of file
