@@ -2,19 +2,22 @@
 
 package com.metallic.chiaki.stream
 
-import androidx.lifecycle.LiveData
-import androidx.lifecycle.MutableLiveData
-import androidx.lifecycle.ViewModel
+import android.app.Application
+import android.content.Context
+import androidx.lifecycle.*
 import com.metallic.chiaki.common.LogManager
 import com.metallic.chiaki.session.StreamSession
 import com.metallic.chiaki.common.Preferences
 import com.metallic.chiaki.lib.*
 import com.metallic.chiaki.session.StreamInput
 
-class StreamViewModel(val preferences: Preferences, val logManager: LogManager, val connectInfo: ConnectInfo): ViewModel()
+class StreamViewModel(val application: Application, val connectInfo: ConnectInfo): ViewModel()
 {
+	val preferences = Preferences(application)
+	val logManager = LogManager(application)
+
 	private var _session: StreamSession? = null
-	val input = StreamInput(preferences)
+	val input = StreamInput(application, preferences)
 	val session = StreamSession(connectInfo, logManager, preferences.logVerbose, input)
 
 	private var _onScreenControlsEnabled = MutableLiveData<Boolean>(preferences.onScreenControlsEnabled)
