@@ -20,6 +20,7 @@ class SettingsDialog : public QDialog
 
 		QCheckBox *log_verbose_check_box;
 		QComboBox *disconnect_action_combo_box;
+		QCheckBox *dualsense_check_box;
 
 		QComboBox *resolution_combo_box;
 		QComboBox *fps_combo_box;
@@ -37,6 +38,7 @@ class SettingsDialog : public QDialog
 
 	private slots:
 		void LogVerboseChanged();
+		void DualSenseChanged();
 		void DisconnectActionSelected();
 
 		void ResolutionSelected();
