@@ -20,6 +20,7 @@
 #include "sessionlog.h"
 #include "controllermanager.h"
 #include "settings.h"
+#include "transformmode.h"
 
 #include <QObject>
 #include <QImage>
@@ -53,9 +54,18 @@ struct StreamSessionConnectInfo
 	ChiakiConnectVideoProfile video_profile;
 	unsigned int audio_buffer_size;
 	bool fullscreen;
+	TransformMode transform_mode;
 	bool enable_keyboard;
+	bool enable_dualsense;
 
-	StreamSessionConnectInfo(Settings *settings, ChiakiTarget target, QString host, QByteArray regist_key, QByteArray morning, bool fullscreen);
+	StreamSessionConnectInfo(
+			Settings *settings,
+			ChiakiTarget target,
+			QString host,
+			QByteArray regist_key,
+			QByteArray morning,
+			bool fullscreen,
+			TransformMode transform_mode);
 };
 
 class StreamSession : public QObject
@@ -92,17 +102,23 @@ class StreamSession : public QObject
 		unsigned int audio_buffer_size;
 		QAudioOutput *audio_output;
 		QIODevice *audio_io;
+		SDL_AudioDeviceID haptics_output;
+		uint8_t *haptics_resampler_buf;
 
 		QMap<Qt::Key, int> key_map;
 
 		void PushAudioFrame(int16_t *buf, size_t samples_count);
+		void PushHapticsFrame(uint8_t *buf, size_t buf_size);
 #if CHIAKI_GUI_ENABLE_SETSU
 		void HandleSetsuEvent(SetsuEvent *event);
 #endif
 
 	private slots:
 		void InitAudio(unsigned int channels, unsigned int rate);
+		void InitHaptics();
 		void Event(ChiakiEvent *event);
+		void DisconnectHaptics();
+		void ConnectHaptics();
 
 	public:
 		explicit StreamSession(const StreamSessionConnectInfo &connect_info, QObject *parent = nullptr);
@@ -124,7 +140,7 @@ class StreamSession : public QObject
 #endif
 
 		void HandleKeyboardEvent(QKeyEvent *event);
-		void HandleMouseEvent(QMouseEvent *event);
+		bool HandleMouseEvent(QMouseEvent *event);
 
 	signals:
 		void FfmpegFrameAvailable();
