@@ -71,7 +71,11 @@ int real_main(int argc, char *argv[])
 
 	QApplication app(argc, argv);
 
+#ifdef Q_OS_MACOS
+	QApplication::setWindowIcon(QIcon(":/icons/chiaki_macos.svg"));
+#else
 	QApplication::setWindowIcon(QIcon(":/icons/chiaki.svg"));
+#endif
 
 	QApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
 
@@ -99,9 +103,15 @@ int real_main(int argc, char *argv[])
 	QCommandLineOption morning_option("morning", "", "morning");
 	parser.addOption(morning_option);
 
-	QCommandLineOption fullscreen_option("fullscreen", "Start window in fullscreen (only for use with stream command)");
+	QCommandLineOption fullscreen_option("fullscreen", "Start window in fullscreen mode [maintains aspect ratio, adds black bars to fill unsused parts of screen if applicable] (only for use with stream command)");
 	parser.addOption(fullscreen_option);
 
+	QCommandLineOption zoom_option("zoom", "Start window in fullscreen zoomed in to fit screen [maintains aspect ratio, cutting off edges of image to fill screen] (only for use with stream command)");
+	parser.addOption(zoom_option);
+
+	QCommandLineOption stretch_option("stretch", "Start window in fullscreen stretched to fit screen [distorts aspect ratio to fill screen] (only for use with stream command)");
+	parser.addOption(stretch_option);
+
 	parser.process(app);
 	QStringList args = parser.positionalArguments();
 
@@ -129,15 +139,21 @@ int real_main(int argc, char *argv[])
 		{
 			if(args.length() < 3)
 				parser.showHelp(1);
+
+			bool found = false;
 			for(const auto &temphost : settings.GetRegisteredHosts())
 			{
 				if(temphost.GetServerNickname() == args[1])
 				{
+					found = true;
 					morning = temphost.GetRPKey();
 					regist_key = temphost.GetRPRegistKey();
 					target = temphost.GetTarget();
 					break;
 				}
+			}
+			if(!found)
+			{
 				printf("No configuration found for '%s'\n", args[1].toLocal8Bit().constData());
 				return 1;
 			}
@@ -164,7 +180,21 @@ int real_main(int argc, char *argv[])
 				return 1;
 			}
 		}
-		StreamSessionConnectInfo connect_info(&settings, target, host, regist_key, morning, parser.isSet(fullscreen_option));
+		if ((parser.isSet(stretch_option) && (parser.isSet(zoom_option) || parser.isSet(fullscreen_option))) || (parser.isSet(zoom_option) && parser.isSet(fullscreen_option)))
+		{
+			printf("Must choose between fullscreen, zoom or stretch option.");
+			return 1;
+		}
+
+		StreamSessionConnectInfo connect_info(
+				&settings,
+				target,
+				host,
+				regist_key,
+				morning,
+				parser.isSet(fullscreen_option),
+				parser.isSet(zoom_option) ? TransformMode::Zoom : parser.isSet(stretch_option) ? TransformMode::Stretch : TransformMode::Fit);
+
 		return RunStream(app, connect_info);
 	}
 #ifdef CHIAKI_ENABLE_CLI
