@@ -146,6 +146,11 @@ MainWindow::MainWindow(Settings *settings, QWidget *parent)
 	AddToolBarAction(settings_action);
 	connect(settings_action, &QAction::triggered, this, &MainWindow::ShowSettings);
 
+	auto quit_action = new QAction(tr("Quit"), this);
+	quit_action->setShortcut(Qt::CTRL + Qt::Key_Q);
+	addAction(quit_action);
+	connect(quit_action, &QAction::triggered, this, &MainWindow::Quit);
+
 	auto scroll_area = new QScrollArea(this);
 	scroll_area->setWidgetResizable(true);
 	scroll_area->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
@@ -249,7 +254,14 @@ void MainWindow::ServerItemWidgetTriggered()
 		}
 
 		QString host = server.GetHostAddr();
-		StreamSessionConnectInfo info(settings, server.registered_host.GetTarget(), host, server.registered_host.GetRPRegistKey(), server.registered_host.GetRPKey(), false);
+		StreamSessionConnectInfo info(
+				settings,
+				server.registered_host.GetTarget(),
+				host,
+				server.registered_host.GetRPRegistKey(),
+				server.registered_host.GetRPKey(),
+				false,
+				TransformMode::Fit);
 		new StreamWindow(info);
 	}
 	else
@@ -298,6 +310,11 @@ void MainWindow::ShowSettings()
 	dialog.exec();
 }
 
+void MainWindow::Quit()
+{
+	qApp->exit();
+}
+
 void MainWindow::UpdateDisplayServers()
 {
 	display_servers.clear();
