@@ -78,6 +78,7 @@ typedef struct chiaki_connect_info_t
 	ChiakiConnectVideoProfile video_profile;
 	bool video_profile_auto_downgrade; // Downgrade video_profile if server does not seem to support it.
 	bool enable_keyboard;
+	bool enable_dualsense;
 } ChiakiConnectInfo;
 
 
@@ -93,11 +94,17 @@ typedef enum {
 	CHIAKI_QUIT_REASON_CTRL_CONNECT_FAILED,
 	CHIAKI_QUIT_REASON_CTRL_CONNECTION_REFUSED,
 	CHIAKI_QUIT_REASON_STREAM_CONNECTION_UNKNOWN,
-	CHIAKI_QUIT_REASON_STREAM_CONNECTION_REMOTE_DISCONNECTED
+	CHIAKI_QUIT_REASON_STREAM_CONNECTION_REMOTE_DISCONNECTED,
+	CHIAKI_QUIT_REASON_STREAM_CONNECTION_REMOTE_SHUTDOWN, // like REMOTE_DISCONNECTED, but because the server shut down
 } ChiakiQuitReason;
 
 CHIAKI_EXPORT const char *chiaki_quit_reason_string(ChiakiQuitReason reason);
 
+static inline bool chiaki_quit_reason_is_error(ChiakiQuitReason reason)
+{
+	return reason != CHIAKI_QUIT_REASON_STOPPED && reason != CHIAKI_QUIT_REASON_STREAM_CONNECTION_REMOTE_SHUTDOWN;
+}
+
 typedef struct chiaki_quit_event_t
 {
 	ChiakiQuitReason reason;
@@ -121,6 +128,14 @@ typedef struct chiaki_rumble_event_t
 	uint8_t right; // high-frequency
 } ChiakiRumbleEvent;
 
+typedef struct chiaki_trigger_effects_event_t
+{
+	uint8_t type_left;
+	uint8_t type_right;
+	uint8_t left[10];
+	uint8_t right[10];
+} ChiakiTriggerEffectsEvent;
+
 typedef enum {
 	CHIAKI_EVENT_CONNECTED,
 	CHIAKI_EVENT_LOGIN_PIN_REQUEST,
@@ -129,6 +144,7 @@ typedef enum {
 	CHIAKI_EVENT_KEYBOARD_REMOTE_CLOSE,
 	CHIAKI_EVENT_RUMBLE,
 	CHIAKI_EVENT_QUIT,
+	CHIAKI_EVENT_TRIGGER_EFFECTS,
 } ChiakiEventType;
 
 typedef struct chiaki_event_t
@@ -139,6 +155,7 @@ typedef struct chiaki_event_t
 		ChiakiQuitEvent quit;
 		ChiakiKeyboardEvent keyboard;
 		ChiakiRumbleEvent rumble;
+		ChiakiTriggerEffectsEvent trigger_effects;
 		struct
 		{
 			bool pin_incorrect; // false on first request, true if the pin entered before was incorrect
@@ -170,6 +187,7 @@ typedef struct chiaki_session_t
 		ChiakiConnectVideoProfile video_profile;
 		bool video_profile_auto_downgrade;
 		bool enable_keyboard;
+		bool enable_dualsense;
 	} connect_info;
 
 	ChiakiTarget target;
@@ -191,6 +209,7 @@ typedef struct chiaki_session_t
 	ChiakiVideoSampleCallback video_sample_cb;
 	void *video_sample_cb_user;
 	ChiakiAudioSink audio_sink;
+	ChiakiAudioSink haptics_sink;
 
 	ChiakiThread session_thread;
 
@@ -246,6 +265,14 @@ static inline void chiaki_session_set_audio_sink(ChiakiSession *session, ChiakiA
 	session->audio_sink = *sink;
 }
 
+/**
+ * @param sink contents are copied
+ */
+static inline void chiaki_session_set_haptics_sink(ChiakiSession *session, ChiakiAudioSink *sink)
+{
+	session->haptics_sink = *sink;
+}
+
 #ifdef __cplusplus
 }
 #endif
