@@ -158,6 +158,8 @@ CHIAKI_EXPORT const char *chiaki_quit_reason_string(ChiakiQuitReason reason)
 			return "Unknown Error in Stream Connection";
 		case CHIAKI_QUIT_REASON_STREAM_CONNECTION_REMOTE_DISCONNECTED:
 			return "Remote has disconnected from Stream Connection";
+		case CHIAKI_QUIT_REASON_STREAM_CONNECTION_REMOTE_SHUTDOWN:
+			return "Remote has disconnected from Stream Connection the because Server shut down";
 		case CHIAKI_QUIT_REASON_NONE:
 		default:
 			return "Unknown";
@@ -227,6 +229,7 @@ CHIAKI_EXPORT ChiakiErrorCode chiaki_session_init(ChiakiSession *session, Chiaki
 	session->connect_info.video_profile = connect_info->video_profile;
 	session->connect_info.video_profile_auto_downgrade = connect_info->video_profile_auto_downgrade;
 	session->connect_info.enable_keyboard = connect_info->enable_keyboard;
+	session->connect_info.enable_dualsense = connect_info->enable_dualsense;
 
 	return CHIAKI_ERR_SUCCESS;
 error_stop_pipe:
@@ -504,7 +507,10 @@ ctrl_failed:
 	if(err == CHIAKI_ERR_DISCONNECTED)
 	{
 		CHIAKI_LOGE(session->log, "Remote disconnected from StreamConnection");
-		session->quit_reason = CHIAKI_QUIT_REASON_STREAM_CONNECTION_REMOTE_DISCONNECTED;
+		if(!strcmp(session->stream_connection.remote_disconnect_reason, "Server shutting down"))
+			session->quit_reason = CHIAKI_QUIT_REASON_STREAM_CONNECTION_REMOTE_SHUTDOWN;
+		else
+			session->quit_reason = CHIAKI_QUIT_REASON_STREAM_CONNECTION_REMOTE_DISCONNECTED;
 		session->quit_reason_str = strdup(session->stream_connection.remote_disconnect_reason);
 	}
 	else if(err != CHIAKI_ERR_SUCCESS && err != CHIAKI_ERR_CANCELED)
