@@ -6,9 +6,10 @@ cd $(dirname "${BASH_SOURCE[0]}")/..
 cd "./$1"
 ROOT="`pwd`"
 
-URL=https://www.libsdl.org/release/SDL2-2.0.10.tar.gz
-FILE=SDL2-2.0.10.tar.gz
-DIR=SDL2-2.0.10
+SDL_VER=2.26.1
+URL=https://www.libsdl.org/release/SDL2-${SDL_VER}.tar.gz
+FILE=SDL2-${SDL_VER}.tar.gz
+DIR=SDL2-${SDL_VER}
 
 if [ ! -d "$DIR" ]; then
 	curl -L "$URL" -O
@@ -21,14 +22,14 @@ mkdir -p build && cd build || exit 1
 cmake \
 	-DCMAKE_INSTALL_PREFIX="$ROOT/sdl2-prefix" \
 	-DSDL_ATOMIC=OFF \
-	-DSDL_AUDIO=OFF \
+	-DSDL_AUDIO=ON \
 	-DSDL_CPUINFO=OFF \
 	-DSDL_EVENTS=ON \
 	-DSDL_FILE=OFF \
 	-DSDL_FILESYSTEM=OFF \
 	-DSDL_HAPTIC=ON \
 	-DSDL_JOYSTICK=ON \
-	-DSDL_LOADSO=OFF \
+	-DSDL_LOADSO=ON \
 	-DSDL_RENDER=OFF \
 	-DSDL_SHARED=ON \
 	-DSDL_STATIC=OFF \
