@@ -1,11 +1,5 @@
 // SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL
 
-#ifdef __SWITCH__
-#include <switch.h>
-#else
-#include <iostream>
-#endif
-
 #include "io.h"
 #include "settings.h"
 
@@ -291,6 +285,16 @@ void IO::AudioCB(int16_t *buf, size_t samples_count)
 		else
 			buf[x] = (int16_t)sample;
 	}
+
+	int audio_queued_size = SDL_GetQueuedAudioSize(this->sdl_audio_device_id);
+	if(audio_queued_size > 16000)
+	{
+		// clear audio queue to avoid big audio delay
+		// average values are close to 13000 bytes
+		CHIAKI_LOGW(this->log, "Triggering SDL_ClearQueuedAudio with queue size = %d", audio_queued_size);
+		SDL_ClearQueuedAudio(this->sdl_audio_device_id);
+	}
+
 	int success = SDL_QueueAudio(this->sdl_audio_device_id, buf, sizeof(int16_t) * samples_count * 2);
 	if(success != 0)
 		CHIAKI_LOGE(this->log, "SDL_QueueAudio failed: %s\n", SDL_GetError());
@@ -335,91 +339,53 @@ bool IO::FreeVideo()
 	return ret;
 }
 
-bool IO::ReadUserKeyboard(char *buffer, size_t buffer_size)
-{
-#ifndef __SWITCH__
-	// use cin to get user input from linux
-	std::cin.getline(buffer, buffer_size);
-	CHIAKI_LOGI(this->log, "Got user input: %s\n", buffer);
-#else
-	// https://kvadevack.se/post/nintendo-switch-virtual-keyboard/
-	SwkbdConfig kbd;
-	Result rc = swkbdCreate(&kbd, 0);
-
-	if(R_SUCCEEDED(rc))
-	{
-		swkbdConfigMakePresetDefault(&kbd);
-		rc = swkbdShow(&kbd, buffer, buffer_size);
-
-		if(R_SUCCEEDED(rc))
-		{
-			CHIAKI_LOGI(this->log, "Got user input: %s\n", buffer);
-		}
-		else
-		{
-			CHIAKI_LOGE(this->log, "swkbdShow() error: %u\n", rc);
-			return false;
-		}
-		swkbdClose(&kbd);
-	}
-	else
-	{
-		CHIAKI_LOGE(this->log, "swkbdCreate() error: %u\n", rc);
-		return false;
-	}
-#endif
-	return true;
-}
-
-bool IO::ReadGameTouchScreen(ChiakiControllerState *state)
+bool IO::ReadGameTouchScreen(ChiakiControllerState *chiaki_state, std::map<uint32_t, int8_t> *finger_id_touch_id)
 {
 #ifdef __SWITCH__
-	hidScanInput();
-	int touch_count = hidTouchCount();
+	HidTouchScreenState sw_state = {0};
+
 	bool ret = false;
-	if(!touch_count)
+	hidGetTouchScreenStates(&sw_state, 1);
+	// scale switch screen to the PS trackpad
+	chiaki_state->buttons &= ~CHIAKI_CONTROLLER_BUTTON_TOUCHPAD; // touchscreen release
+
+	// un-touch all old touches
+	for(auto it = finger_id_touch_id->begin(); it != finger_id_touch_id->end();)
 	{
-		for(int i = 0; i < CHIAKI_CONTROLLER_TOUCHES_MAX; i++)
+		auto cur = it;
+		it++;
+		for(int i = 0; i < sw_state.count; i++)
 		{
-			if(state->touches[i].id != -1)
-			{
-				state->touches[i].x = 0;
-				state->touches[i].y = 0;
-				state->touches[i].id = -1;
-				state->buttons &= ~CHIAKI_CONTROLLER_BUTTON_TOUCHPAD; // touchscreen release
-				// the state changed
-				ret = true;
-			}
+			if(sw_state.touches[i].finger_id == cur->first)
+				goto cont;
 		}
-		return ret;
+		if(cur->second >= 0)
+			chiaki_controller_state_stop_touch(chiaki_state, (uint8_t)cur->second);
+		finger_id_touch_id->erase(cur);
+cont:
+		continue;
 	}
 
-	touchPosition touch;
-	for(int i = 0; i < touch_count && i < CHIAKI_CONTROLLER_TOUCHES_MAX; i++)
+
+	// touch or update all current touches
+	for(int i = 0; i < sw_state.count; i++)
 	{
-		hidTouchRead(&touch, i);
+		uint16_t x = sw_state.touches[i].x * ((float)DS4_TRACKPAD_MAX_X / (float)SWITCH_TOUCHSCREEN_MAX_X);
+		uint16_t y = sw_state.touches[i].y * ((float)DS4_TRACKPAD_MAX_Y / (float)SWITCH_TOUCHSCREEN_MAX_Y);
+		// use nintendo switch border's 5% to trigger the touchpad button
+		if(x <= (DS4_TRACKPAD_MAX_X * 0.05) || x >= (DS4_TRACKPAD_MAX_X * 0.95) || y <= (DS4_TRACKPAD_MAX_Y * 0.05) || y >= (DS4_TRACKPAD_MAX_Y * 0.95))
+			chiaki_state->buttons |= CHIAKI_CONTROLLER_BUTTON_TOUCHPAD; // touchscreen
 
-		// 1280×720 px (16:9)
-		// ps4 controller aspect ratio looks closer to 29:10
-		uint16_t x = touch.px * (DS4_TRACKPAD_MAX_X / SWITCH_TOUCHSCREEN_MAX_X);
-		uint16_t y = touch.py * (DS4_TRACKPAD_MAX_Y / SWITCH_TOUCHSCREEN_MAX_Y);
-
-		// use nintendo switch border's 5% to
-		if(x <= (SWITCH_TOUCHSCREEN_MAX_X * 0.05) || x >= (SWITCH_TOUCHSCREEN_MAX_X * 0.95) || y <= (SWITCH_TOUCHSCREEN_MAX_Y * 0.05) || y >= (SWITCH_TOUCHSCREEN_MAX_Y * 0.95))
+		auto it = finger_id_touch_id->find(sw_state.touches[i].finger_id);
+		if(it == finger_id_touch_id->end())
 		{
-			state->buttons |= CHIAKI_CONTROLLER_BUTTON_TOUCHPAD; // touchscreen
-																 // printf("CHIAKI_CONTROLLER_BUTTON_TOUCHPAD\n");
+			// new touch
+			(*finger_id_touch_id)[sw_state.touches[i].finger_id] =
+				chiaki_controller_state_start_touch(chiaki_state, x, y);
 		}
-		else
-		{
-			state->buttons &= ~CHIAKI_CONTROLLER_BUTTON_TOUCHPAD; // touchscreen release
-		}
-
-		state->touches[i].x = x;
-		state->touches[i].y = y;
-		state->touches[i].id = i;
-		// printf("[point_id=%d] px=%03d, py=%03d, dx=%03d, dy=%03d, angle=%03d\n",
-		// i, touch.px, touch.py, touch.dx, touch.dy, touch.angle);
+		else if(it->second >= 0)
+			chiaki_controller_state_set_touch_pos(chiaki_state, (uint8_t)it->second, x, y);
+		// it->second < 0 ==> touch ignored because there were already too many multi-touches
 		ret = true;
 	}
 	return ret;
@@ -428,14 +394,143 @@ bool IO::ReadGameTouchScreen(ChiakiControllerState *state)
 #endif
 }
 
+void IO::SetRumble(uint8_t left, uint8_t right)
+{
+#ifdef __SWITCH__
+	Result rc = 0;
+	HidVibrationValue vibration_values[] = {
+		{
+			.amp_low = 0.0f,
+			.freq_low = 160.0f,
+			.amp_high = 0.0f,
+			.freq_high = 320.0f,
+		},
+		{
+			.amp_low = 0.0f,
+			.freq_low = 160.0f,
+			.amp_high = 0.0f,
+			.freq_high = 320.0f,
+		}};
+
+	int target_device = padIsHandheld(&pad) ? 0 : 1;
+	if(left > 0)
+	{
+		// SDL_HapticRumblePlay(this->sdl_haptic_ptr[0], left / 100, 5000);
+		vibration_values[0].amp_low = (float)left / (float)100;
+		vibration_values[0].amp_high = (float)left / (float)100;
+		vibration_values[0].freq_low *= (float)left / (float)100;
+		vibration_values[0].freq_high *= (float)left / (float)100;
+	}
+
+	if(right > 0)
+	{
+		// SDL_HapticRumblePlay(this->sdl_haptic_ptr[1], right / 100, 5000);
+		vibration_values[1].amp_low = (float)right / (float)100;
+		vibration_values[1].amp_high = (float)right / (float)100;
+		vibration_values[1].freq_low *= (float)left / (float)100;
+		vibration_values[1].freq_high *= (float)left / (float)100;
+	}
+
+	// printf("left ptr %p amp_low %f amp_high %f freq_low %f freq_high %f\n",
+	// 	&vibration_values[0],
+	// 	vibration_values[0].amp_low,
+	// 	vibration_values[0].amp_high,
+	// 	vibration_values[0].freq_low,
+	// 	vibration_values[0].freq_high);
+
+	// printf("right ptr %p amp_low %f amp_high %f freq_low %f freq_high %f\n",
+	// 	&vibration_values[1],
+	// 	vibration_values[1].amp_low,
+	// 	vibration_values[1].amp_high,
+	// 	vibration_values[1].freq_low,
+	// 	vibration_values[1].freq_high);
+
+	rc = hidSendVibrationValues(this->vibration_handles[target_device], vibration_values, 2);
+	if(R_FAILED(rc))
+		CHIAKI_LOGE(this->log, "hidSendVibrationValues() returned: 0x%x", rc);
+
+#endif
+}
+
+bool IO::ReadGameSixAxis(ChiakiControllerState *state)
+{
+#ifdef __SWITCH__
+	// Read from the correct sixaxis handle depending on the current input style
+	HidSixAxisSensorState sixaxis = {0};
+	uint64_t style_set = padGetStyleSet(&pad);
+	if(style_set & HidNpadStyleTag_NpadHandheld)
+		hidGetSixAxisSensorStates(this->sixaxis_handles[0], &sixaxis, 1);
+	else if(style_set & HidNpadStyleTag_NpadFullKey)
+		hidGetSixAxisSensorStates(this->sixaxis_handles[1], &sixaxis, 1);
+	else if(style_set & HidNpadStyleTag_NpadJoyDual)
+	{
+		// For JoyDual, read from either the Left or Right Joy-Con depending on which is/are connected
+		u64 attrib = padGetAttributes(&pad);
+		if(attrib & HidNpadAttribute_IsLeftConnected)
+			hidGetSixAxisSensorStates(this->sixaxis_handles[2], &sixaxis, 1);
+		else if(attrib & HidNpadAttribute_IsRightConnected)
+			hidGetSixAxisSensorStates(this->sixaxis_handles[3], &sixaxis, 1);
+	}
+
+	state->gyro_x = sixaxis.angular_velocity.x * 2.0f * M_PI;
+	state->gyro_y = sixaxis.angular_velocity.z * 2.0f * M_PI;
+	state->gyro_z = -sixaxis.angular_velocity.y * 2.0f * M_PI;
+	state->accel_x = -sixaxis.acceleration.x;
+	state->accel_y = -sixaxis.acceleration.z;
+	state->accel_z = sixaxis.acceleration.y;
+
+	// https://d3cw3dd2w32x2b.cloudfront.net/wp-content/uploads/2015/01/matrix-to-quat.pdf
+	float (*dm)[3] = sixaxis.direction.direction;
+	float m[3][3] = {
+		{ dm[0][0], dm[2][0], dm[1][0] },
+		{ dm[0][2], dm[2][2], dm[1][2] },
+		{ dm[0][1], dm[2][1], dm[1][1] }
+	};
+	std::array<float, 4> q;
+	float t;
+	if(m[2][2] < 0)
+	{
+		if (m[0][0] > m[1][1])
+		{
+			t = 1 + m[0][0] - m[1][1] - m[2][2];
+			q = { t, m[0][1] + m[1][0], m[2][0] + m[0][2], m[1][2] - m[2][1] };
+		}
+		else
+		{
+			t = 1 - m[0][0] + m[1][1] -m[2][2];
+			q = { m[0][1] + m[1][0], t, m[1][2] + m[2][1], m[2][0] - m[0][2] };
+		}
+	}
+	else
+	{
+		if(m[0][0] < -m[1][1])
+		{
+			t = 1 - m[0][0] - m[1][1] + m[2][2];
+			q = { m[2][0] + m[0][2], m[1][2] + m[2][1], t, m[0][1] - m[1][0] };
+		}
+		else
+		{
+			t = 1 + m[0][0] + m[1][1] + m[2][2];
+			q = { m[1][2] - m[2][1], m[2][0] - m[0][2], m[0][1] - m[1][0], t };
+		}
+	}
+	float fac = 0.5f / sqrt(t);
+	state->orient_x = q[0] * fac;
+	state->orient_y = q[1] * fac;
+	state->orient_z = -q[2] * fac;
+	state->orient_w = q[3] * fac;
+	return true;
+#else
+	return false;
+#endif
+}
+
 bool IO::ReadGameKeys(SDL_Event *event, ChiakiControllerState *state)
 {
 	// return true if an event changed (gamepad input)
 
 	// TODO
 	// share vs PS button
-	// Gyro ?
-	// rumble ?
 	bool ret = true;
 	switch(event->type)
 	{
@@ -812,7 +907,7 @@ inline void IO::OpenGlDraw()
 	D(glFinish());
 }
 
-bool IO::InitJoystick()
+bool IO::InitController()
 {
 	// https://github.com/switchbrew/switch-examples/blob/master/graphics/sdl2/sdl2-simple/source/main.cpp#L57
 	// open CONTROLLER_PLAYER_1 and CONTROLLER_PLAYER_2
@@ -826,24 +921,64 @@ bool IO::InitJoystick()
 			CHIAKI_LOGE(this->log, "SDL_JoystickOpen: %s\n", SDL_GetError());
 			return false;
 		}
+		// this->sdl_haptic_ptr[i] = SDL_HapticOpenFromJoystick(sdl_joystick_ptr[i]);
+		// SDL_HapticRumbleInit(this->sdl_haptic_ptr[i]);
+		// if(sdl_haptic_ptr[i] == nullptr)
+		// {
+		// 	CHIAKI_LOGE(this->log, "SDL_HapticRumbleInit: %s\n", SDL_GetError());
+		// }
 	}
+#ifdef __SWITCH__
+Result rc = 0;
+	// Configure our supported input layout: a single player with standard controller styles
+	padConfigureInput(1, HidNpadStyleSet_NpadStandard);
+
+	// Initialize the default gamepad (which reads handheld mode inputs as well as the first connected controller)
+	padInitializeDefault(&this->pad);
+	// touchpad
+	hidInitializeTouchScreen();
+	// It's necessary to initialize these separately as they all have different handle values
+	hidGetSixAxisSensorHandles(&this->sixaxis_handles[0], 1, HidNpadIdType_Handheld, HidNpadStyleTag_NpadHandheld);
+	hidGetSixAxisSensorHandles(&this->sixaxis_handles[1], 1, HidNpadIdType_No1, HidNpadStyleTag_NpadFullKey);
+	hidGetSixAxisSensorHandles(&this->sixaxis_handles[2], 2, HidNpadIdType_No1, HidNpadStyleTag_NpadJoyDual);
+	hidStartSixAxisSensor(this->sixaxis_handles[0]);
+	hidStartSixAxisSensor(this->sixaxis_handles[1]);
+	hidStartSixAxisSensor(this->sixaxis_handles[2]);
+	hidStartSixAxisSensor(this->sixaxis_handles[3]);
+
+    rc = hidInitializeVibrationDevices(this->vibration_handles[0], 2, HidNpadIdType_Handheld, HidNpadStyleTag_NpadHandheld);
+	if(R_FAILED(rc))
+		CHIAKI_LOGE(this->log, "hidInitializeVibrationDevices() HidNpadIdType_Handheld returned: 0x%x", rc);
+
+    rc = hidInitializeVibrationDevices(this->vibration_handles[1], 2, HidNpadIdType_No1, HidNpadStyleTag_NpadJoyDual);
+	if(R_FAILED(rc))
+		CHIAKI_LOGE(this->log, "hidInitializeVibrationDevices() HidNpadIdType_No1 returned: 0x%x", rc);
+
+#endif
 	return true;
 }
 
-bool IO::FreeJoystick()
+bool IO::FreeController()
 {
 	for(int i = 0; i < SDL_JOYSTICK_COUNT; i++)
 	{
-		if(SDL_JoystickGetAttached(sdl_joystick_ptr[i]))
-			SDL_JoystickClose(sdl_joystick_ptr[i]);
+		SDL_JoystickClose(this->sdl_joystick_ptr[i]);
+		// SDL_HapticClose(this->sdl_haptic_ptr[i]);
 	}
+#ifdef __SWITCH__
+	hidStopSixAxisSensor(this->sixaxis_handles[0]);
+	hidStopSixAxisSensor(this->sixaxis_handles[1]);
+	hidStopSixAxisSensor(this->sixaxis_handles[2]);
+	hidStopSixAxisSensor(this->sixaxis_handles[3]);
+#endif
 	return true;
 }
 
-bool IO::MainLoop(ChiakiControllerState *state)
+void IO::UpdateControllerState(ChiakiControllerState *state, std::map<uint32_t, int8_t> *finger_id_touch_id)
 {
-	D(glUseProgram(this->prog));
-
+#ifdef __SWITCH__
+	padUpdate(&this->pad);
+#endif
 	// handle SDL events
 	while(SDL_PollEvent(&this->sdl_event))
 	{
@@ -851,11 +986,17 @@ bool IO::MainLoop(ChiakiControllerState *state)
 		switch(this->sdl_event.type)
 		{
 			case SDL_QUIT:
-				return false;
+				this->quit = true;
 		}
 	}
 
-	ReadGameTouchScreen(state);
+	ReadGameTouchScreen(state, finger_id_touch_id);
+	ReadGameSixAxis(state);
+}
+
+bool IO::MainLoop()
+{
+	D(glUseProgram(this->prog));
 
 	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
 	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
